@@ -128,8 +128,8 @@ var gStatusFlags = {GOODSIG:         nsIEnigmail.GOOD_SIGNATURE,
                     SC_OP_FAILURE:   nsIEnigmail.SC_OP_FAILURE,
                     UNKNOWN_ALGO:    nsIEnigmail.UNKNOWN_ALGO,
                     SIG_CREATED:     nsIEnigmail.SIG_CREATED,
-                    END_ENCRYPTION : nsIEnigmail.END_ENCRYPTION,
-                    INV_SGNR:				 0x100000000
+                    END_ENCRYPTION:  nsIEnigmail.END_ENCRYPTION,
+                    INV_SGNR:        0x100000000,
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -144,7 +144,7 @@ const NS_RDONLY      = 0x01;
 const NS_WRONLY      = 0x02;
 const NS_CREATE_FILE = 0x08;
 const NS_TRUNCATE    = 0x20;
-const DEFAULT_FILE_PERMS = 0600;
+const DEFAULT_FILE_PERMS = 0x180; // equals 0600
 
 const ENC_TYPE_ATTACH_BINARY = 1;
 const ENC_TYPE_ATTACH_ASCII = 2;
@@ -153,70 +153,6 @@ const DUMMY_AGENT_INFO = "none";
 
 var gKeyAlgorithms = [];
 
-
-function CreateFileStream(filePath, permissions) {
-
-  //Ec.DEBUG_LOG("enigmail.js: CreateFileStream: file="+filePath+"\n");
-
-  try {
-    var localFile;
-    if (typeof filePath == "string") {
-      localFile = Cc[NS_LOCAL_FILE_CONTRACTID].createInstance(Ci.nsIFile);
-      initPath(localFile, filePath);
-    }
-    else {
-      localFile = filePath.QueryInterface(Ci.nsIFile);
-    }
-
-    if (localFile.exists()) {
-
-      if (localFile.isDirectory() || !localFile.isWritable())
-         throw Components.results.NS_ERROR_FAILURE;
-
-      if (!permissions)
-        permissions = localFile.permissions;
-    }
-
-    if (!permissions)
-      permissions = DEFAULT_FILE_PERMS;
-
-    var flags = NS_WRONLY | NS_CREATE_FILE | NS_TRUNCATE;
-
-    var fileStream = Cc[NS_LOCALFILEOUTPUTSTREAM_CONTRACTID].createInstance(Ci.nsIFileOutputStream);
-
-    fileStream.init(localFile, flags, permissions, 0);
-
-    return fileStream;
-
-  } catch (ex) {
-    Ec.ERROR_LOG("enigmail.js: CreateFileStream: Failed to create "+filePath+"\n");
-    return null;
-  }
-}
-
-function WriteFileContents(filePath, data, permissions) {
-
-  Ec.DEBUG_LOG("enigmail.js: WriteFileContents: file="+filePath.toString()+"\n");
-
-  try {
-    var fileOutStream = CreateFileStream(filePath, permissions);
-
-    if (data.length) {
-      if (fileOutStream.write(data, data.length) != data.length)
-        throw Components.results.NS_ERROR_FAILURE;
-
-      fileOutStream.flush();
-    }
-    fileOutStream.close();
-
-  } catch (ex) {
-    Ec.ERROR_LOG("enigmail.js: WriteFileContents: Failed to write to "+filePath+"\n");
-    return false;
-  }
-
-  return true;
-}
-
 // Read the contents of a file into a string
 
 function readFile(filePath) {
@@ -468,6 +404,7 @@ function IndexOfArmorDelimiter(text, str, offset) {
 function Enigmail()
 {
   Components.utils.import("resource://enigmail/enigmailCommon.jsm");
+  Components.utils.import("resource://enigmail/commonFuncs.jsm");
   Ec = EnigmailCommon;
   EnigmailGpgAgent.setEnigmailCommon(Ec);
 
@@ -613,7 +550,7 @@ Enigmail.prototype = {
     var prefix = this.getLogDirectoryPrefix();
     if (prefix) {
       gLogLevel = 5;
-      this.logFileStream = CreateFileStream(prefix+"enigdbug.txt");
+      this.logFileStream = EnigmailFuncs.createFileStream(prefix+"enigdbug.txt");
       Ec.DEBUG_LOG("enigmail.js: Logging debug output to "+prefix+"enigdbug.txt\n");
     }
 
@@ -999,12 +936,14 @@ Enigmail.prototype = {
       // env. variable suggests running gpg-agent
       this.gpgAgentInfo.preStarted = true;
       this.gpgAgentInfo.envStr = gpgAgentInfo;
+      Ec.gpgAgentIsOptional = false;
     }
     else {
       Ec.DEBUG_LOG("enigmail.js: detectGpgAgent: no GPG_AGENT_INFO variable set\n");
       this.gpgAgentInfo.preStarted = false;
 
       if (this.agentVersion >= "2.0") {
+        Ec.gpgAgentIsOptional = false;
         if (this.agentVersion >= "2.0.16") {
           Ec.DEBUG_LOG("enigmail.js: detectGpgAgent: gpg 2.0.16 or newer - not starting agent\n");
         }
@@ -1161,8 +1100,8 @@ Enigmail.prototype = {
     var prefix = this.getLogDirectoryPrefix();
     if (prefix && (gLogLevel >= 4)) {
 
-      WriteFileContents(prefix+"enigcmd.txt", Ec.printCmdLine(command, args)+"\n");
-      WriteFileContents(prefix+"enigenv.txt", envList.join(",")+"\n");
+      EnigmailFuncs.writeFileContents(prefix+"enigcmd.txt", Ec.printCmdLine(command, args)+"\n");
+      EnigmailFuncs.writeFileContents(prefix+"enigenv.txt", envList.join(",")+"\n");
 
       Ec.DEBUG_LOG("enigmail.js: Enigmail.simpleExecCmd: copied command line/env/input to files "+prefix+"enigcmd.txt/enigenv.txt/eniginp.txt\n");
     }
@@ -1194,8 +1133,8 @@ Enigmail.prototype = {
        errorMsgObj.value  = errOutput;
 
     if (prefix && (gLogLevel >= 4)) {
-      WriteFileContents(prefix+"enigout.txt", outputData);
-      WriteFileContents(prefix+"enigerr.txt", errOutput);
+      EnigmailFuncs.writeFileContents(prefix+"enigout.txt", outputData);
+      EnigmailFuncs.writeFileContents(prefix+"enigerr.txt", errOutput);
       Ec.DEBUG_LOG("enigmail.js: Enigmail.simpleExecCmd: copied command out/err data to files "+prefix+"enigout.txt/enigerr.txt\n");
     }
 
@@ -1232,13 +1171,13 @@ Enigmail.prototype = {
 
       if (prependPassphrase) {
         // Obscure passphrase
-        WriteFileContents(prefix+"eniginp.txt", "<passphrase>"+input);
+        EnigmailFuncs.writeFileContents(prefix+"eniginp.txt", "<passphrase>"+input);
       } else {
-        WriteFileContents(prefix+"eniginp.txt", input);
+        EnigmailFuncs.writeFileContents(prefix+"eniginp.txt", input);
       }
 
-      WriteFileContents(prefix+"enigcmd.txt", Ec.printCmdLine(command, args)+"\n");
-      WriteFileContents(prefix+"enigenv.txt", envList.join(",")+"\n");
+      EnigmailFuncs.writeFileContents(prefix+"enigcmd.txt", Ec.printCmdLine(command, args)+"\n");
+      EnigmailFuncs.writeFileContents(prefix+"enigenv.txt", envList.join(",")+"\n");
 
       Ec.DEBUG_LOG("enigmail.js: Enigmail.execCmd: copied command line/env/input to files "+prefix+"enigcmd.txt/enigenv.txt/eniginp.txt\n");
     }
@@ -1285,8 +1224,8 @@ Enigmail.prototype = {
     if (proc.errorData) errOutput  = proc.errorData;
 
     if (prefix && (gLogLevel >= 4)) {
-      WriteFileContents(prefix+"enigout.txt", outputData);
-      WriteFileContents(prefix+"enigerr.txt", errOutput);
+      EnigmailFuncs.writeFileContents(prefix+"enigout.txt", outputData);
+      EnigmailFuncs.writeFileContents(prefix+"enigerr.txt", errOutput);
       Ec.DEBUG_LOG("enigmail.js: Enigmail.execCmd: copied command out/err data to files "+prefix+"enigout.txt/enigerr.txt\n");
     }
 
@@ -1314,8 +1253,9 @@ Enigmail.prototype = {
   },
 
 
-  encryptMessage: function (parent, uiFlags, plainText, fromMailAddr, toMailAddr, bccMailAddr,
-            sendFlags, exitCodeObj, statusFlagsObj, errorMsgObj) {
+  encryptMessage: function (parent, uiFlags, plainText, fromMailAddr, toMailAddr, bccMailAddr, sendFlags,
+                            exitCodeObj, statusFlagsObj, errorMsgObj)
+  {
     Ec.DEBUG_LOG("enigmail.js: Enigmail.encryptMessage: "+plainText.length+" bytes from "+fromMailAddr+" to "+toMailAddr+" ("+sendFlags+")\n");
 
     exitCodeObj.value    = -1;
@@ -1354,10 +1294,10 @@ Enigmail.prototype = {
       });
 
 
-    var proc = Ec.encryptMessageStart(parent, uiFlags, fromMailAddr, toMailAddr,
-                          bccMailAddr, null, sendFlags,
-                          listener, statusFlagsObj, errorMsgObj);
-
+    var proc = Ec.encryptMessageStart(parent, uiFlags,
+                                      fromMailAddr, toMailAddr, bccMailAddr,
+                                      null, sendFlags,
+                                      listener, statusFlagsObj, errorMsgObj);
     if (! proc) {
       exitCodeObj.value = -1;
       return "";
@@ -1368,9 +1308,9 @@ Enigmail.prototype = {
 
     var retStatusObj = {};
     exitCodeObj.value = Ec.encryptMessageEnd(getUnicodeData(listener.stderrData), listener.exitCode,
-                                        uiFlags, sendFlags,
-                                        listener.stdoutData.length,
-                                        retStatusObj);
+                                             uiFlags, sendFlags,
+                                             listener.stdoutData.length,
+                                             retStatusObj);
 
     statusFlagsObj.value = retStatusObj.statusFlags;
     errorMsgObj.value = retStatusObj.errorMsg;
@@ -1381,13 +1321,11 @@ Enigmail.prototype = {
 
     if (exitCodeObj.value == 0) {
       // Normal return
-
       return getUnicodeData(listener.stdoutData);
     }
 
     // Error processing
     Ec.DEBUG_LOG("enigmail.js: Enigmail.encryptMessage: command execution exit code: "+exitCodeObj.value+"\n");
-
     return "";
   },
 
@@ -1527,6 +1465,45 @@ Enigmail.prototype = {
     return "";
   },
 
+  statusObjectFrom: function (signatureObj, exitCodeObj, statusFlagsObj, keyIdObj, userIdObj, sigDetailsObj, errorMsgObj, blockSeparationObj) {
+    return {
+      signature: signatureObj,
+      exitCode: exitCodeObj,
+      statusFlags: statusFlagsObj,
+      keyId: keyIdObj,
+      userId: userIdObj,
+      signatureDetails: sigDetailsObj,
+      message: errorMsgObj,
+      blockSeparation: blockSeparationObj
+    };
+  },
+
+  newStatusObject: function () {
+    return this.statusObjectFrom({value: ""}, {}, {}, {}, {}, {}, {}, {});
+  },
+
+  mergeStatusInto: function(left, right) {
+    left.statusFlags.value = left.statusFlags.value | right.statusFlags.value;
+    left.keyId.value = right.keyId.value;
+    left.userId.value = right.userId.value;
+    left.signatureDetails.value = right.signatureDetails.value;
+    left.message.value = right.message.value;
+  },
+
+  inlineInnerVerification: function (parent, uiFlags, text, statusObject) {
+    Ec.DEBUG_LOG("enigmail.js: Enigmail.inlineInnerVerification\n");
+
+    if (text && text.indexOf("-----BEGIN PGP SIGNED MESSAGE-----") == 0) {
+      var status = this.newStatusObject();
+      var newText = this.decryptMessage(parent, uiFlags, text, status.signature, status.exitCode, status.statusFlags, status.keyId, status.userId, status.signatureDetails, status.message, status.blockSeparation);
+      if (status.exitCode.value == 0) {
+        text = newText;
+        this.mergeStatusInto(statusObject, status);
+      }
+    }
+
+    return text;
+  },
 
   decryptMessage: function (parent, uiFlags, cipherText, signatureObj, exitCodeObj,
             statusFlagsObj, keyIdObj, userIdObj, sigDetailsObj, errorMsgObj,
@@ -1634,8 +1611,11 @@ Enigmail.prototype = {
           pipe.close();
       });
 
+    var maxOutput = pgpBlock.length * 100;  // limit output to 100 times message size
+                                            // to avoid DoS attack
     var proc = Ec.decryptMessageStart(parent, verifyOnly, noOutput, listener,
-                                      statusFlagsObj, startErrorMsgObj);
+                                      statusFlagsObj, startErrorMsgObj,
+                                      null, maxOutput);
 
     if (!proc) {
       errorMsgObj.value = startErrorMsgObj.value;
@@ -1689,7 +1669,9 @@ Enigmail.prototype = {
         plainText = plainText.replace(/^/g, indentStrObj.value);
         RegExp.multiline = false;
       }
-      return plainText;
+
+      return this.inlineInnerVerification(parent, uiFlags, plainText,
+                        this.statusObjectFrom(signatureObj, exitCodeObj, statusFlagsObj, keyIdObj, userIdObj, sigDetailsObj, errorMsgObj, blockSeparationObj));
     }
 
     var pubKeyId = keyIdObj.value;
@@ -1797,7 +1779,7 @@ Enigmail.prototype = {
       errorMsgObj.value = Ec.getString("failKeyExtract");
 
       if (cmdErrorMsgObj.value) {
-        errorMsgObj.value += "\n" + command;
+        errorMsgObj.value += "\n" + Ec.printCmdLine(this.agentPath, args);
         errorMsgObj.value += "\n" + cmdErrorMsgObj.value;
       }
 
@@ -1819,7 +1801,7 @@ Enigmail.prototype = {
         errorMsgObj.value = Ec.getString("failKeyExtract");
 
         if (cmdErrorMsgObj.value) {
-          errorMsgObj.value += "\n" + command;
+          errorMsgObj.value += "\n" + Ec.printCmdLine(this.agentPath, args);;
           errorMsgObj.value += "\n" + cmdErrorMsgObj.value;
         }
 
@@ -1831,7 +1813,7 @@ Enigmail.prototype = {
     }
 
     if (outputFile) {
-      if (! WriteFileContents(outputFile, keyBlock, DEFAULT_FILE_PERMS)) {
+      if (! EnigmailFuncs.writeFileContents(outputFile, keyBlock, DEFAULT_FILE_PERMS)) {
         exitCodeObj.value = -1;
         errorMsgObj.value = Ec.getString("fileWriteFailed", [ outputFile ]);
       }
@@ -1996,6 +1978,7 @@ Enigmail.prototype = {
 
   invalidateUserIdList: function () {
     // clean the userIdList to force reloading the list at next usage
+    Ec.DEBUG_LOG("enigmail.js: Enigmail.invalidateUserIdList\n");
     this.userIdList= null;
   },
 
@@ -2084,8 +2067,21 @@ Enigmail.prototype = {
     return listText;
   },
 
-  getKeyDetails: function (keyId, uidOnly) {
-      // uidOnly==true also means to only show UIDs with highest trust level
+  /**
+   * Return details of given keys.
+   *
+   * @param  String  keyId              List of keys with 0x, separated by spaces.
+   * @param  Boolean uidOnly            false:
+   *                                      return all key details (full output of GnuPG)
+   *                                    true:
+   *                                      return only the user ID fields. Only UIDs with highest trust
+   *                                      level are returned.
+   * @param  Boolean withUserAttributes true: if uidOnly include "uat:jpegPhoto" (but not subkey IDs)
+   *
+   * @return String all key details or list of user IDs separated by \n.
+   */
+  getKeyDetails: function (keyId, uidOnly, withUserAttributes)
+  {
     var args = Ec.getAgentArgs(true);
     var keyIdList = keyId.split(" ");
     args=args.concat([ "--fixed-list-mode", "--with-colons", "--list-keys"]);
@@ -2097,43 +2093,61 @@ Enigmail.prototype = {
     var exitCodeObj = {};
 
     var listText = this.execCmd(this.agentPath, args, null, "",
-                      exitCodeObj, statusFlagsObj, statusMsgObj, cmdErrorMsgObj);
+                                exitCodeObj, statusFlagsObj, statusMsgObj, cmdErrorMsgObj);
     if (exitCodeObj.value != 0) {
       return "";
     }
     listText=listText.replace(/(\r\n|\r)/g, "\n");
 
-    const trustLevels = "oidre-qmnfu";
+    const TRUSTLEVELS_SORTED = EnigmailFuncs.trustlevelsSorted();
     var maxTrustLevel = -1;
 
     if (uidOnly) {
       var userList="";
       var hideInvalidUid=true;
-      var keyArr=listText.split(/\n/);
-      for (var i=0; i<keyArr.length; i++) {
-        switch (keyArr[i].substr(0,4)) {
-        case "pub:":
-          if ("idre".indexOf(keyArr[i].split(/:/)[1]) >= 0) {
-            // pub key not valid (anymore)-> display all UID's
-            hideInvalidUid = false;
-          }
-        case "uid:":
-          var theLine=keyArr[i].split(/:/);
-          if (uidOnly && hideInvalidUid) {
-            var thisTrust = trustLevels.indexOf(theLine[1]);
-            if (thisTrust > maxTrustLevel) {
-              userList = theLine[9] + "\n";
-              maxTrustLevel = thisTrust;
+      var lineArr=listText.split(/\n/);
+      for (var i=0; i<lineArr.length; i++) {
+        // process lines such as:
+        //  tru::1:1395895453:1442881280:3:1:5
+        //  pub:f:4096:1:C1B875ED336XX959:2299509307:1546189300::f:::scaESCA:
+        //  fpr:::::::::102A1C8CC524A966849C33D7C8B157EA336XX959:
+        //  uid:f::::1388511201::67D5B96DC564598D4D4D9E0E89F5B83C9931A154::Joe Fox <joe@fox.com>:
+        //  sig:::1:C8B157EA336XX959:2299509307::::Joe Fox <joe@fox.com>:13x:::::2:
+        //  sub:e:2048:1:B214734F0F5C7041:1316219469:1199912694:::::e:
+        //  sub:f:2048:1:70E7A471DABE08B0:1316221524:1546189300:::::s:
+        var lineTokens = lineArr[i].split(/:/);
+        switch (lineTokens[0]) {
+          case "pub":
+            if (EnigmailFuncs.isInvalid(lineTokens[1])) {
+              // pub key not valid (anymore)-> display all UID's
+              hideInvalidUid = false;
             }
-            else if (thisTrust == maxTrustLevel) {
-              userList += theLine[9] + "\n";
+            break;
+          case "uid":
+            if (uidOnly && hideInvalidUid) {
+              var thisTrust = TRUSTLEVELS_SORTED.indexOf(lineTokens[1]);
+              if (thisTrust > maxTrustLevel) {
+                userList = lineTokens[9] + "\n";
+                maxTrustLevel = thisTrust;
+              }
+              else if (thisTrust == maxTrustLevel) {
+                userList += lineTokens[9] + "\n";
+              }
+              // else do not add uid
             }
-            // else do not add uid
-          }
-          else if (("idre".indexOf(theLine[1]) < 0) || (! hideInvalidUid)) {
-            // UID valid or key not valid
-            userList += theLine[9] + "\n";
-          }
+            else if (!EnigmailFuncs.isInvalid(lineTokens[1]) || !hideInvalidUid) {
+              // UID valid  OR  key not valid, but invalid keys allowed
+              userList += lineTokens[9] + "\n";
+            }
+            break;
+          case "uat":
+            if (withUserAttributes) {
+              if (!EnigmailFuncs.isInvalid(lineTokens[1]) || !hideInvalidUid) {
+                // IF  UID valid  OR  key not valid and invalid keys allowed
+                userList += "uat:jpegPhoto:" + lineTokens[4] + "\n";
+              }
+            }
+            break;
         }
       }
       return userList.replace(/^\n+/, "").replace(/\n+$/, "").replace(/\n\n+/g, "\n");
@@ -2142,9 +2156,55 @@ Enigmail.prototype = {
     return listText;
   },
 
-  // returns the output of --with-colons --list-config
-  getGnupgConfig: function  (exitCodeObj, errorMsgObj) {
 
+  /**
+   * Return first found userId of given keys.
+   * @param  String  keyId key with leading 0x
+   * @return String  First found of user IDs or null if none
+   */
+  getFirstUserIdOfKey: function (keyId)
+  {
+    var args = Ec.getAgentArgs(true);
+    args=args.concat([ "--fixed-list-mode", "--with-colons", "--list-keys"]);
+    args=args.concat([keyId]);
+
+    var statusMsgObj   = {};
+    var cmdErrorMsgObj = {};
+    var statusFlagsObj = {};
+    var exitCodeObj = {};
+
+    var listText = this.execCmd(this.agentPath, args, null, "",
+                                exitCodeObj, statusFlagsObj, statusMsgObj, cmdErrorMsgObj);
+    if (exitCodeObj.value != 0) {
+      return null;
+    }
+    listText=listText.replace(/(\r\n|\r)/g, "\n");
+
+    var lineArr=listText.split(/\n/);
+    for (var i=0; i<lineArr.length; i++) {
+      // process lines such as:
+      //  tru::1:1395895453:1442881280:3:1:5
+      //  pub:f:4096:1:C1B875ED336XX959:2299509307:1546189300::f:::scaESCA:
+      //  fpr:::::::::102A1C8CC524A966849C33D7C8B157EA336XX959:
+      //  uid:f::::1388511201::67D5B96DC564598D4D4D9E0E89F5B83C9931A154::Joe Fox <joe@fox.com>:
+      //  sig:::1:C8B157EA336XX959:2299509307::::Joe Fox <joe@fox.com>:13x:::::2:
+      var lineTokens = lineArr[i].split(/:/);
+      switch (lineTokens[0]) {
+        case "uid":
+          {
+            var userId = lineTokens[9];
+            return userId;
+          }
+          break;
+      }
+    }
+    return null;
+  },
+
+
+  // returns the output of --with-colons --list-config
+  getGnupgConfig: function  (exitCodeObj, errorMsgObj)
+  {
     var args = Ec.getAgentArgs(true);
 
     args=args.concat(["--fixed-list-mode", "--with-colons", "--list-config"]);
@@ -2494,7 +2554,7 @@ Enigmail.prototype = {
     if (rulesFile) {
       if (this.rulesList) {
         // the rule list is not empty -> write into file
-        return WriteFileContents(rulesFile.path,
+        return EnigmailFuncs.writeFileContents(rulesFile.path,
                                domSerializer.serializeToString(this.rulesList.firstChild),
                                DEFAULT_FILE_PERMS);
       }
