@@ -67,8 +67,15 @@ Enigmail.hdrView = {
     this.statusBar = document.getElementById("enigmail-status-bar");
     this.enigmailBox = document.getElementById("enigmailBox");
 
+    var addrPopup = document.getElementById("emailAddressPopup");
+    if (addrPopup) {
+      var attr = addrPopup.getAttribute("onpopupshowing");
+      attr = "EnigmailFuncs.collapseAdvanced(this, 'hidden'); "+attr;
+      addrPopup.setAttribute("onpopupshowing", attr);
+    }
   },
 
+
   statusBarHide: function ()
   {
     try {
@@ -110,7 +117,7 @@ Enigmail.hdrView = {
   },
 
 
-  updateHdrIcons: function (exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation)
+  updateHdrIcons: function (exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation, xtraStatus)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: this.updateHdrIcons: exitCode="+exitCode+", statusFlags="+statusFlags+", keyId="+keyId+", userId="+userId+", "+errorMsg+"\n");
 
@@ -197,29 +204,30 @@ Enigmail.hdrView = {
     }
 
     var msgSigned = (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
-              nsIEnigmail.GOOD_SIGNATURE |
-              nsIEnigmail.EXPIRED_KEY_SIGNATURE |
-              nsIEnigmail.EXPIRED_SIGNATURE |
-              nsIEnigmail.UNVERIFIED_SIGNATURE |
-              nsIEnigmail.REVOKED_KEY |
-              nsIEnigmail.EXPIRED_KEY_SIGNATURE |
-              nsIEnigmail.EXPIRED_SIGNATURE));
+                                    nsIEnigmail.GOOD_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_KEY_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_SIGNATURE |
+                                    nsIEnigmail.UNVERIFIED_SIGNATURE |
+                                    nsIEnigmail.REVOKED_KEY |
+                                    nsIEnigmail.EXPIRED_KEY_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_SIGNATURE));
     var msgEncrypted = (statusFlags & (nsIEnigmail.DECRYPTION_OKAY |
-              nsIEnigmail.DECRYPTION_INCOMPLETE |
-              nsIEnigmail.DECRYPTION_FAILED));
+                                       nsIEnigmail.DECRYPTION_INCOMPLETE |
+                                       nsIEnigmail.DECRYPTION_FAILED));
 
     if (msgSigned && (statusFlags & nsIEnigmail.IMPORTED_KEY)) {
       statusFlags &= (~nsIEnigmail.IMPORTED_KEY);
     }
 
     if (((!(statusFlags & (nsIEnigmail.DECRYPTION_INCOMPLETE |
-              nsIEnigmail.DECRYPTION_FAILED |
-              nsIEnigmail.UNVERIFIED_SIGNATURE |
-              nsIEnigmail.BAD_SIGNATURE))) ||
+                           nsIEnigmail.DECRYPTION_FAILED |
+                           nsIEnigmail.UNVERIFIED_SIGNATURE |
+                           nsIEnigmail.BAD_SIGNATURE)))
+         ||
          (statusFlags & nsIEnigmail.DISPLAY_MESSAGE) &&
           !(statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE)) &&
             !(statusFlags & nsIEnigmail.IMPORTED_KEY)) {
-      // Normal exit / display message
+      // normal exit / display message
       statusLine = errorMsg;
       statusInfo = statusLine;
 
@@ -243,94 +251,110 @@ Enigmail.hdrView = {
       }
       fullStatusInfo = statusInfo;
 
-    } else {
-      if (keyId) {
-        statusInfo = EnigmailCommon.getString("keyNeeded", [ keyId ]);
-
-        if (statusFlags & nsIEnigmail.INLINE_KEY) {
-          statusLine = statusInfo + EnigmailCommon.getString("clickDecrypt");
-        } else {
-          statusLine = statusInfo + EnigmailCommon.getString("clickPen");
-        }
-
-        statusInfo = EnigmailCommon.getString("unverifiedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickPen");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
-        statusInfo = EnigmailCommon.getString("unverifiedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickQueryPenDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
-                                nsIEnigmail.UNVERIFIED_SIGNATURE |
-                                nsIEnigmail.EXPIRED_SIGNATURE |
-                                nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
-        statusInfo = EnigmailCommon.getString("failedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickPenDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
-        statusInfo = EnigmailCommon.getString("incompleteDecrypt");
-        statusLine = statusInfo + EnigmailCommon.getString("clickKey");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
+    }
+    else {
+      // no normal exit / don't display message
+      // - process failed decryptions first because they imply bad signature handling
+      if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
         if (statusFlags & nsIEnigmail.NO_SECKEY) {
           statusInfo = EnigmailCommon.getString("needKey");
         } else {
           statusInfo = EnigmailCommon.getString("failedDecrypt");
         }
-
         statusLine = statusInfo + EnigmailCommon.getString("clickKeyDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
+      }
+      else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
         statusInfo = EnigmailCommon.getString("badPhrase");
         statusLine = statusInfo + EnigmailCommon.getString("clickDecryptRetry");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
+      }
+      else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+        statusInfo = EnigmailCommon.getString("unverifiedSig");
+        statusLine = statusInfo + EnigmailCommon.getString("clickQueryPenDetails");
+      }
+      else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
+                              nsIEnigmail.EXPIRED_SIGNATURE |
+                              nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
+        statusInfo = EnigmailCommon.getString("failedSig");
+        statusLine = statusInfo + EnigmailCommon.getString("clickPenDetails");
+      }
+      else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
+        statusInfo = EnigmailCommon.getString("incompleteDecrypt");
+        statusLine = statusInfo + EnigmailCommon.getString("clickKey");
+      }
+      else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
         statusLine = "";
         statusInfo = "";
         EnigmailCommon.alert(window, errorMsg);
-
-      } else {
+      }
+      else {
         statusInfo = EnigmailCommon.getString("failedDecryptVerify");
         statusLine = statusInfo + EnigmailCommon.getString("viewInfo");
-        statusInfo += "\n\n" + errorMsg;
       }
+      // add key infos if available
+      if (keyId) {
+        var si = EnigmailCommon.getString("unverifiedSig");  // "Unverified signature"
+        if (statusInfo == "") {
+          statusInfo += si;
+          statusLine = si + EnigmailCommon.getString("clickPen");
+        }
+        //if (statusFlags & nsIEnigmail.INLINE_KEY) {
+        //  statusLine = statusInfo + EnigmailCommon.getString("clickDecrypt");
+        //} else {
+        //  statusLine = statusInfo + EnigmailCommon.getString("clickPen");
+        //}
+        statusInfo += "\n" + EnigmailCommon.getString("keyNeeded", [ keyId ]);  // "public key ... needed"
+      }
+      statusInfo += "\n\n" + errorMsg;
     }
 
     if (statusFlags & nsIEnigmail.DECRYPTION_OKAY ||
         (this.statusBar.getAttribute("encrypted")=="ok")) {
-      if (!statusInfo) {
-        statusInfo = EnigmailCommon.getString("decryptedMsg");
+      var statusMsg;
+      if (xtraStatus && xtraStatus == "buggyMailFormat") {
+        statusMsg = EnigmailCommon.getString("decryptedMsgWithFormatError");
       }
       else {
-        statusInfo = EnigmailCommon.getString("decryptedMsg")+"\n"+statusInfo;
+        statusMsg = EnigmailCommon.getString("decryptedMsg");
+      }
+      if (!statusInfo) {
+        statusInfo = statusMsg;
+      }
+      else {
+        statusInfo = statusMsg + "\n" + statusInfo;
       }
       if (!statusLine) {
-        statusLine=statusInfo;
+        statusLine = statusInfo;
       }
       else {
-        statusLine=EnigmailCommon.getString("decryptedMsg")+"; "+statusLine;
+        statusLine = statusMsg + "; " + statusLine;
       }
     }
 
     if (EnigmailCommon.getPref("displayPartiallySigned")) {
       if (statusFlags & nsIEnigmail.PARTIALLY_PGP) {
         if (msgSigned && msgEncrypted) {
-          statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSignedAndEnc") ]);
-          statusLine += EnigmailCommon.getString("clickPenKeyDetails");
+          if (statusLine == "") {
+            statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSignedAndEnc") ]);
+            statusLine += EnigmailCommon.getString("clickPenKeyDetails");
+          }
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ])
+                        + "\n" + statusInfo;
         }
         else if (msgEncrypted) {
-          statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgEncrypted") ]);
-          statusLine += EnigmailCommon.getString("clickQueryKeyDetails");
+          if (statusLine == "") {
+            statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgEncrypted") ]);
+            statusLine += EnigmailCommon.getString("clickQueryKeyDetails");
+          }
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ])
+                        + "\n" + statusInfo;
         }
         else if (msgSigned) {
-          statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ]);
-          statusLine += EnigmailCommon.getString("clickQueryPenDetails");
+          if (statusLine == "") {
+            statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ]);
+            statusLine += EnigmailCommon.getString("clickQueryPenDetails");
+          }
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ])
+                        + "\n" + statusInfo;
         }
       }
     }
@@ -433,8 +457,15 @@ Enigmail.hdrView = {
         this.statusBar.setAttribute("encrypted", "notok");
         this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelSignatureNotOk");
       }
+
+      // special handling after trying to fix buggy mail format (see buggyExchangeEmailContent in code)
+      if (xtraStatus && xtraStatus == "buggyMailFormat") {
+        this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelBuggyMailFormat");
+      }
+
       this.updateMsgDb();
 
+
     } catch (ex) {}
   },
 
@@ -831,6 +862,7 @@ Enigmail.hdrView = {
       }
     }
   }
+
 };
 
 window.addEventListener("load", Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView), false);
