@@ -1,4 +1,6 @@
-/*
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
  * The contents of this file are subject to the Mozilla Public
  * License Version 1.1 (the "MPL"); you may not use this file
  * except in compliance with the MPL. You may obtain a copy of
@@ -11,27 +13,24 @@
  *
  * The Original Code is Enigmail.
  *
- * The Initial Developer of this code is Patrick Brunschwig.
- * Portions created by Patrick Brunschwig <patrick.brunschwig@gmx.net>
- * are Copyright (C) 2004 Patrick Brunschwig.
- * All Rights Reserved.
+ * The Initial Developer of the Original Code is Patrick Brunschwig.
+ * Portions created by Patrick Brunschwig <patrick@enigmail.net> are
+ * Copyright (C) 2003 Patrick Brunschwig. All Rights Reserved.
  *
  * Contributor(s):
  *
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License (the "GPL") or the GNU 
- * Lesser General Public License (the "LGPL"), in which case
- * the provisions of the GPL or the LGPL are applicable instead of
- * those above. If you wish to allow use of your version of this
- * file only under the terms of the GPL or the LGPL and not to allow
- * others to use your version of this file under the MPL, indicate
- * your decision by deleting the provisions above and replace them
- * with the notice and other provisions required by the GPL or the 
- * LGPL respectively.
- * If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL, the
- * GPL or the LGPL.
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ * ***** END LICENSE BLOCK ***** */
 
 // Uses: chrome://enigmail/content/enigmailCommon.js
 
@@ -40,10 +39,8 @@ EnigInitCommon("enigmailRulesEditor");
 
 const INPUT=0;
 const RESULT=1;
- 
-var gSearchTimer = null;
+
 var gSearchInput = null;
-var gClearButton = null;
 var gNumRows = null;
 
 function enigmailDlgOnLoad() {
@@ -55,8 +52,8 @@ function enigmailDlgOnLoad() {
   if (enigmailSvc.getRulesData(rulesListObj)) {
     var treeChildren=document.getElementById("rulesTreeChildren");
     var rulesList=rulesListObj.value;
-        if (rulesList.firstChild.nodeName=="parsererror") {
-      EnigAlert("Invalid enigmail.xml file:\n"+ rulesList.firstChild.textContent);
+    if (rulesList.firstChild.nodeName=="parsererror") {
+      EnigAlert("Invalid pgprules.xml file:\n"+ rulesList.firstChild.textContent);
       return;
     }
     DEBUG_LOG("enigmailRulesEditor.js: dlgOnLoad: keys loaded\n");
@@ -65,7 +62,7 @@ function enigmailDlgOnLoad() {
     while (node) {
       if (node.tagName=="pgpRule") {
         //try {
-          var userObj={ 
+          var userObj={
                 email: node.getAttribute("email"),
                 keyId: node.getAttribute("keyId"),
                 sign: node.getAttribute("sign"),
@@ -76,7 +73,7 @@ function enigmailDlgOnLoad() {
           if (node.getAttribute("negateRule")) {
             userObj.negate=node.getAttribute("negateRule");
           }
-          
+
           var treeItem=document.createElement("treeitem");
           createRow(treeItem, userObj);
           treeChildren.appendChild(treeItem);
@@ -88,7 +85,6 @@ function enigmailDlgOnLoad() {
   }
   var rulesTree=document.getElementById("rulesTree");
   gSearchInput = document.getElementById("filterEmail");
-  gClearButton = document.getElementById("clearButton");
 
 }
 
@@ -98,7 +94,7 @@ function enigmailDlgOnAccept() {
   if (!enigmailSvc)
     return false;
   enigmailSvc.clearRules();
-  
+
   var node=getFirstNode();
   while(node) {
     enigmailSvc.addRule(true,
@@ -112,7 +108,7 @@ function enigmailDlgOnAccept() {
     node = node.nextSibling;
   }
   enigmailSvc.saveRulesFile();
-  
+
   return true;
 }
 
@@ -125,13 +121,13 @@ function createCol(value, label, treeItem, translate) {
   case "encrypt":
   case "pgpMime":
     switch (Number(label)) {
-    case 0: 
+    case 0:
       label=EnigGetString("never");
       break;
-    case 1: 
+    case 1:
       label=EnigGetString("possible");
       break;
-    case 2: 
+    case 2:
       label=EnigGetString("always");
       break;
     }
@@ -164,11 +160,11 @@ function createRow(treeItem, userObj) {
   treeRow.appendChild(negate);
   treeRow.appendChild(email);
   treeRow.appendChild(keyId);
-  treeRow.appendChild(sign);
   treeRow.appendChild(encrypt);
+  treeRow.appendChild(sign);
   treeRow.appendChild(pgpMime);
   treeRow.setAttribute("rowId", ++gNumRows);
-  
+
   if (treeItem.firstChild) {
     treeItem.replaceChild(treeRow, treeItem.firstChild);
   }
@@ -180,15 +176,15 @@ function createRow(treeItem, userObj) {
 function getFirstNode() {
   return document.getElementById("rulesTreeChildren").firstChild;
 }
-  
+
 function getCurrentNode() {
   var rulesTree=document.getElementById("rulesTree");
-  return rulesTree.contentView.getItemAtIndex(rulesTree.currentIndex)
+  return rulesTree.contentView.getItemAtIndex(rulesTree.currentIndex);
 }
 
 
 function enigDoEdit() {
-  var node=getCurrentNode()
+  var node=getCurrentNode();
   if (node) {
     var inputObj  = new Object;
     var resultObj = new Object;
@@ -200,7 +196,7 @@ function enigDoEdit() {
     inputObj.encrypt   = Number(node.getAttribute("encrypt"));
     inputObj.pgpmime   = Number(node.getAttribute("pgpMime"));
     inputObj.negate    = Number(node.getAttribute("negateRule"));
-    
+
     window.openDialog("chrome://enigmail/content/enigmailSingleRcptSettings.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
     if (resultObj.cancelled==false) {
       createRow(node, resultObj);
@@ -214,7 +210,7 @@ function enigDoAdd() {
   inputObj.options = "nosave";
   inputObj.toAddress = "{}";
   inputObj.command = "add";
-  
+
   window.openDialog("chrome://enigmail/content/enigmailSingleRcptSettings.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
   if (resultObj.cancelled==false) {
     var treeItem=document.createElement("treeitem");
@@ -232,7 +228,7 @@ function enigDoAdd() {
 function enigDoDelete() {
   var node=getCurrentNode();
   if (node) {
-    if (EnigConfirm(EnigGetString("deleteRule"))) {
+    if (EnigConfirm(EnigGetString("deleteRule"), EnigGetString("dlg.button.delete"))) {
       var treeChildren=document.getElementById("rulesTreeChildren");
       treeChildren.removeChild(node);
     }
@@ -293,55 +289,16 @@ function enigDoResetFilter() {
     node.hidden=false;
     node = node.nextSibling;
   }
-  gClearButton.setAttribute("disabled", true);
 }
 
-function onEnterInSearchBar()
+function onSearchInput()
 {
-   if (gSearchInput.value == "") 
+   if (gSearchInput.value == "")
    {
      enigDoResetFilter();
      return;
    }
 
-   gClearButton.setAttribute("disabled", false);
-
    enigDoSearch();
 }
 
-
-
-function onSearchKeyPress(event)
-{
-  // 13 == return
-  if (event && event.keyCode == 13) {
-    event.stopPropagation(); // make sure the dialog is not closed...
-    onSearchInput(true);
-  }
-}
-
-
-function onSearchInput(returnKeyHit)
-{
-  if (gSearchTimer) {
-    clearTimeout(gSearchTimer); 
-    gSearchTimer = null;
-  }
-
-  // only select the text when the return key was hit
-  if (returnKeyHit) {
-    gSearchInput.select();
-    onEnterInSearchBar();
-  }
-  else {
-    gSearchTimer = setTimeout("onEnterInSearchBar();", 800);
-  }
-}
-
-
-function disableQuickSearchClearButton()
-{
- if (gClearButton)
-   gClearButton.setAttribute("disabled", true); 
-}
-
