@@ -56,6 +56,7 @@ const KEY_NOT_VALID=KEY_EXPIRED+KEY_REVOKED+KEY_INVALID+KEY_DISABLED;
 var gErrorData = "";
 var gOutputData = "";
 var gEnigRequest;
+var gAllKeysSelected = 0;
 
 function trim(str) {
   return str.replace(/^(\s*)(.*)/, "$2").replace(/\s+$/,"");
@@ -133,6 +134,28 @@ function onLoad () {
 }
 
 
+function selectAllKeys () {
+  Ec.DEBUG_LOG("enigmailSearchKey.js: selectAllkeys\n");
+  var keySelList = document.getElementById("enigmailKeySel");
+  var treeChildren=keySelList.getElementsByAttribute("id", "enigmailKeySelChildren")[0];
+
+  gEnigRequest.dlKeyList = [];
+
+  // Toggle flag to select/deselect all when hotkey is pressed repeatedly
+  gAllKeysSelected ^= 1;
+
+  var item=treeChildren.firstChild;
+  while (item) {
+    var aRows = item.getElementsByAttribute("id","indicator");
+    if (aRows.length) {
+      var elem=aRows[0];
+      EnigSetActive(elem, gAllKeysSelected);
+    }
+    item = item.nextSibling;
+  }
+}
+
+
 function onAccept () {
   Ec.DEBUG_LOG("enigmailSearchKey.js: onAccept\n");
 
@@ -161,6 +184,7 @@ function startDownload() {
     gEnigRequest.progressMeter.value = 0;
     gEnigRequest.progressMeter.mode = "undetermined";
     document.getElementById("progress.box").removeAttribute("hidden");
+    document.getElementById("selall-button").setAttribute("hidden", "true");
     document.getElementById("dialog.accept").setAttribute("disabled", "true");
     gEnigRequest.keyNum = 0;
     gEnigRequest.errorTxt="";
@@ -216,6 +240,11 @@ function enigStatusError () {
 }
 
 function enigCloseDialog() {
+  if (window.arguments[RESULT].importedKeys > 0) {
+    var enigmailSvc = GetEnigmailSvc();
+    enigmailSvc.invalidateUserIdList();
+  }
+
   document.getElementById("enigmailSearchKeyDlg").cancelDialog();
   window.close();
 }
@@ -386,6 +415,7 @@ function enigScanKeys(connType, htmlTxt) {
   gEnigRequest.httpInProgress=false;
   enigPopulateList(gEnigRequest.keyList);
   document.getElementById("progress.box").setAttribute("hidden", "true");
+  document.getElementById("selall-button").removeAttribute("hidden");
   if (gEnigRequest.keyList.length == 0) {
     Ec.alert(window, Ec.getString("noKeyFound"));
     enigCloseDialog();
@@ -414,7 +444,8 @@ function enigScanHtmlKeys (txt) {
         key={
           keyId: m[2],
           created: m[3],
-          uid: []
+          uid: [],
+          status: ""
         };
         if (m[4].search(/.+<.+@.+>/)>=0) {
           if (! ignoreUid(m[4])) key.uid.push(trim(m[4]));
@@ -478,7 +509,9 @@ function enigScanGpgKeys(txt) {
           key={
             keyId: m[0],
             created: dat.getFullYear()+"-"+month+"-"+day,
-            uid: []
+            uid: [],
+            status: ""
+
           };
           if (! ignoreUid(m[1])) key.uid.push(m[1]);
         }
@@ -499,7 +532,8 @@ function enigScanGpgKeys(txt) {
         key={
           keyId: m[1],
           created: dat.getFullYear()+"-"+month+"-"+day,
-          uid: []
+          uid: [],
+          status: (m.length >= 5 ? m[6] : "")
         };
       }
     }
@@ -586,7 +620,7 @@ function enigNewGpgKeysRequest(requestType, callbackFunction) {
 
 
 function enigmailGpgkeysTerminate(exitCode) {
-  Ec.DEBUG_LOG("enigmailSearchkey.js: Terminate: exitCode="+exitCode+"\n");
+  Ec.DEBUG_LOG("enigmailSearchkey.js: enigmailGpgkeysTerminate: exitCode="+exitCode+"\n");
 
   gEnigRequest.gpgkeysRequest = null;
 
@@ -614,6 +648,23 @@ function enigPopulateList(keyList) {
      if (a.uid[0]<b.uid[0]) { return -1; } else {return 1; }
   };
 
+  var sortKeyIds = function (c,d) {
+       if (c.keyId<d.keyId) { return -1; } else {return 1; }
+  };
+
+  keyList.sort(sortKeyIds);
+
+  // remove duplicates
+  var z = 0;
+  while (z<keyList.length-1) {
+    if (keyList[z].keyId == keyList[z+1].keyId) {
+      keyList.splice(z,1);
+    }
+    else {
+      z = z + 1;
+    }
+  }
+
   keyList.sort(sortUsers);
 
   var treeList = document.getElementById("enigmailKeySel");
@@ -621,12 +672,12 @@ function enigPopulateList(keyList) {
   var treeItem;
 
   for (var i=0; i<keyList.length; i++) {
-    treeItem = enigUserSelCreateRow(keyList[i].keyId, false, keyList[i].uid[0], keyList[i].created, "");
+    treeItem = enigUserSelCreateRow(keyList[i].keyId, false, keyList[i].uid[0], keyList[i].created, keyList[i].status);
     if (keyList[i].uid.length>1) {
       treeItem.setAttribute("container", "true");
       var subChildren=document.createElement("treechildren");
       for (j=1; j<keyList[i].uid.length; j++) {
-        var subItem=enigUserSelCreateRow(keyList[i].keyId, true, keyList[i].uid[j], "", "");
+        var subItem=enigUserSelCreateRow(keyList[i].keyId, true, keyList[i].uid[j], "", keyList[i].status);
         subChildren.appendChild(subItem);
       }
       treeItem.appendChild(subChildren);
@@ -641,12 +692,13 @@ function enigPopulateList(keyList) {
 }
 
 function enigUserSelCreateRow (keyId, subKey, userId, dateField, trustStatus) {
+    Ec.DEBUG_LOG("enigmailSearchKey.js: enigUserSelCreateRow\n");
     var selectCol=document.createElement("treecell");
     selectCol.setAttribute("id", "indicator");
     var expCol=document.createElement("treecell");
     var userCol=document.createElement("treecell");
     userCol.setAttribute("id", "name");
-    if (trustStatus.charAt(0)==KEY_EXPIRED) {
+    if (trustStatus.indexOf(KEY_EXPIRED)>=0) {
       expCol.setAttribute("label", Ec.getString("selKeyExpired", dateField));
     }
     else {
@@ -674,6 +726,20 @@ function enigUserSelCreateRow (keyId, subKey, userId, dateField, trustStatus) {
     userRow.appendChild(keyCol);
     var treeItem=document.createElement("treeitem");
     treeItem.setAttribute("id", "0x"+keyId);
+
+    if (trustStatus.length > 0 && KEY_NOT_VALID.indexOf(trustStatus.charAt(0))>=0) {
+      // key invalid, mark it in grey
+      for (var node=userRow.firstChild; node; node=node.nextSibling) {
+        var attr=node.getAttribute("properties");
+        if (typeof(attr)=="string") {
+          node.setAttribute("properties", attr+" enigKeyInactive");
+        }
+        else {
+          node.setAttribute("properties", "enigKeyInactive");
+        }
+      }
+    }
+
     treeItem.appendChild(userRow);
     return treeItem;
 }
