@@ -43,6 +43,14 @@ EnigInitCommon("pref-enigmail");
 
 var gMimePartsElement, gMimePartsValue, gAdvancedMode;
 
+// saved old manual preferences to switch back
+// to them if we temporarily enabled convenient encryption
+// (not persistent)
+var gSavedManualPrefKeepSettingsForReply = true;
+var gSavedManualPrefAcceptedKeys = 1;
+var gSavedManualPrefAutoSendEncrypted = 1;
+var gSavedManualPrefConfirmBeforeSending = 0;
+
 function displayPrefs(showDefault, showPrefs, setPrefs) {
   DEBUG_LOG("pref-enigmail.js displayPrefs\n");
 
@@ -73,16 +81,13 @@ function displayPrefs(showDefault, showPrefs, setPrefs) {
           if (prefElement.getAttribute("invert") == "true") {
             prefValue = ! prefValue;
           }
-
           if (prefValue) {
             prefElement.setAttribute("checked", "true");
           } else {
             prefElement.removeAttribute("checked");
           }
         }
-
         if (setPrefs) {
-
           if (prefElement.getAttribute("invert") == "true") {
             if (prefElement.checked) {
               EnigSetPref(prefName, false);
@@ -98,13 +103,12 @@ function displayPrefs(showDefault, showPrefs, setPrefs) {
             }
           }
         }
-
         break;
 
       case EnigmailCommon.prefBranch.PREF_INT:
-        if (showPrefs)
+        if (showPrefs) {
           prefElement.value = prefValue;
-
+        }
         if (setPrefs) {
           try {
             EnigSetPref(prefName, 0+prefElement.value);
@@ -113,10 +117,12 @@ function displayPrefs(showDefault, showPrefs, setPrefs) {
         break;
 
       case EnigmailCommon.prefBranch.PREF_STRING:
-        if (showPrefs)
+        if (showPrefs) {
           prefElement.value = prefValue;
-        if (setPrefs)
+        }
+        if (setPrefs) {
           EnigSetPref(prefName, prefElement.value);
+        }
         break;
 
       default:
@@ -126,91 +132,103 @@ function displayPrefs(showDefault, showPrefs, setPrefs) {
   }
 }
 
-function prefOnLoad() {
+function prefOnLoad()
+{
+  DEBUG_LOG("pref-enigmail.js: prefOnLoad()\n");
 
-   GetEnigmailSvc();
-   displayPrefs(false, true, false);
+  GetEnigmailSvc();
+  displayPrefs(false, true, false);
 
-   document.getElementById("enigmail_agentPath").value = EnigConvertToUnicode(EnigGetPref("agentPath"), "utf-8");
+  document.getElementById("enigmail_agentPath").value = EnigConvertToUnicode(EnigGetPref("agentPath"), "utf-8");
 
-   var maxIdle = -1;
-   if (! gEnigmailSvc) {
-     maxIdle = EnigmailCommon.getPref("maxIdleMinutes");
-   }
-   else
-     maxIdle = EnigmailGpgAgent.getMaxIdlePref(window);
+  var maxIdle = -1;
+  if (! gEnigmailSvc) {
+    maxIdle = EnigmailCommon.getPref("maxIdleMinutes");
+  }
+  else {
+    maxIdle = EnigmailGpgAgent.getMaxIdlePref(window);
+  }
 
-   document.getElementById("maxIdleMinutes").value = maxIdle;
-   gAdvancedMode = EnigGetPref("advancedUser");
+  document.getElementById("maxIdleMinutes").value = maxIdle;
+  gAdvancedMode = EnigGetPref("advancedUser");
 
-   if (window.arguments) {
-      if (! window.arguments[0].showBasic) {
-          // hide basic tab
-          document.getElementById("basic").setAttribute("collapsed", true);
-          document.getElementById("basicTab").setAttribute("collapsed", true);
-          selectPrefTabPanel("sendTab");
-      }
-      else {
-        EnigCollapseAdvanced(document.getElementById("prefTabBox"), "collapsed", null);
-        EnigCollapseAdvanced(document.getElementById("enigPrefTabPanel"), "hidden", null);
-        enigShowUserModeButtons(gAdvancedMode);
-      }
+  if (window.arguments) {
+     if (! window.arguments[0].showBasic) {
+         // hide basic tab
+         document.getElementById("basic").setAttribute("collapsed", true);
+         document.getElementById("basicTab").setAttribute("collapsed", true);
+         selectPrefTabPanel("sendingTab");
+     }
+     else {
+       EnigCollapseAdvanced(document.getElementById("prefTabBox"), "collapsed", null);
+       EnigCollapseAdvanced(document.getElementById("enigPrefTabPanel"), "hidden", null);
+       enigShowUserModeButtons(gAdvancedMode);
+     }
 
-      if ((typeof window.arguments[0].selectTab)=="string") {
-          selectPrefTabPanel(window.arguments[0].selectTab);
-      }
+     if ((typeof window.arguments[0].selectTab)=="string") {
+         selectPrefTabPanel(window.arguments[0].selectTab);
+     }
 
-   }
-   else {
-    enigShowUserModeButtons(gAdvancedMode);
-   }
+  }
+  else {
+     enigShowUserModeButtons(gAdvancedMode);
+  }
 
-   if (gEnigmailSvc && gEnigmailSvc.useGpgAgent()) {
-      document.getElementById("enigmail_noPassphrase").setAttribute("collapsed", true);
-      document.getElementById("enigmail_useGpgAgent").setAttribute("collapsed", true);
-   }
+  if (! EnigmailCommon.gpgAgentIsOptional) {
+    document.getElementById("enigmail_noPassphrase").setAttribute("collapsed", true);
+    document.getElementById("enigmail_useGpgAgent").setAttribute("collapsed", true);
+  }
 
-   if ((! window.arguments) || (window.arguments[0].clientType!="seamonkey")) {
-      EnigCollapseAdvanced(document.getElementById("prefTabBox"), "collapsed", null);
-      EnigCollapseAdvanced(document.getElementById("enigPrefTabPanel"), "hidden", null);
+  if ((! window.arguments) || (window.arguments[0].clientType!="seamonkey")) {
+    EnigCollapseAdvanced(document.getElementById("prefTabBox"), "collapsed", null);
+    EnigCollapseAdvanced(document.getElementById("enigPrefTabPanel"), "hidden", null);
+  }
 
-   }
+  // init "saved manual preferences" with current settings:
+  gSavedManualPrefKeepSettingsForReply = EnigGetPref("keepSettingsForReply");
+  gSavedManualPrefAcceptedKeys = EnigGetPref("acceptedKeys");
+  gSavedManualPrefAutoSendEncrypted = EnigGetPref("autoSendEncrypted");
+  gSavedManualPrefConfirmBeforeSending = EnigGetPref("confirmBeforeSending");
+  gEnigEncryptionModel = EnigGetPref("encryptionModel");
+  if (gEnigEncryptionModel == 0) { // convenient encryption
+    resetSendingPrefsConvenient();
+  }
+  else {
+    resetSendingPrefsManually();
+  }
 
-   EnigDisplayRadioPref("recipientsSelection", EnigGetPref("recipientsSelection"),
-                        gEnigRecipientsSelection);
+  gMimePartsElement = document.getElementById("mime_parts_on_demand");
 
-   gMimePartsElement = document.getElementById("mime_parts_on_demand");
+  try {
+    gMimePartsValue = EnigmailCommon.prefRoot.getBoolPref("mail.server.default.mime_parts_on_demand");
+  } catch (ex) {
+    gMimePartsValue = true;
+  }
 
-   try {
-     gMimePartsValue = EnigmailCommon.prefRoot.getBoolPref("mail.server.default.mime_parts_on_demand");
-   } catch (ex) {
-     gMimePartsValue = true;
-   }
+  if (gMimePartsValue) {
+    gMimePartsElement.setAttribute("checked", "true");
+  }
+  else {
+    gMimePartsElement.removeAttribute("checked");
+  }
 
-   if (gMimePartsValue) {
-     gMimePartsElement.setAttribute("checked", "true");
-   } else {
-     gMimePartsElement.removeAttribute("checked");
-   }
+  var overrideGpg = document.getElementById("enigOverrideGpg");
+  if (EnigGetPref("agentPath")) {
+    overrideGpg.checked = true;
+  }
+  else {
+    overrideGpg.checked = false;
+  }
+  enigActivateDependent(overrideGpg, "enigmail_agentPath enigmail_browsePath");
 
-   var overrideGpg = document.getElementById("enigOverrideGpg");
-   if (EnigGetPref("agentPath")) {
-      overrideGpg.checked = true;
-   }
-   else {
-      overrideGpg.checked = false;
-   }
-   enigActivateDependent(overrideGpg, "enigmail_agentPath enigmail_browsePath");
-   activateRulesButton(document.getElementById("enigmail_recipientsSelection"), "openRulesEditor");
+  var testEmailElement = document.getElementById("enigmail_test_email");
+  var userIdValue = EnigGetPref("userIdValue");
 
-   var testEmailElement = document.getElementById("enigmail_test_email");
-   var userIdValue = EnigGetPref("userIdValue");
-
-   enigDetermineGpgPath();
-
-   if (testEmailElement && userIdValue)
-     testEmailElement.value = userIdValue;
+  enigDetermineGpgPath();
 
+  if (testEmailElement && userIdValue) {
+    testEmailElement.value = userIdValue;
+  }
 }
 
 function enigDetermineGpgPath() {
@@ -261,9 +279,107 @@ function resetPrefs() {
 
   EnigSetPref("configuredVersion", EnigGetVersion());
 
-  EnigDisplayRadioPref("recipientsSelection", EnigGetPref("recipientsSelection"),
-                      gEnigRecipientsSelection);
+  // init "saved manual preferences" with current settings:
+  gSavedManualPrefKeepSettingsForReply = EnigGetPref("keepSettingsForReply");
+  gSavedManualPrefAcceptedKeys = EnigGetPref("acceptedKeys");
+  gSavedManualPrefAutoSendEncrypted = EnigGetPref("autoSendEncrypted");
+  gSavedManualPrefConfirmBeforeSending = EnigGetPref("confirmBeforeSending");
+  // and process encryption model:
+  gEnigEncryptionModel = EnigGetPref("encryptionModel");
+  if (gEnigEncryptionModel == 0) { // convenient encryption
+    resetSendingPrefsConvenient();
+  }
+  else {
+    resetSendingPrefsManually();
+  }
+}
 
+function disableManually (disable)
+{
+  var elems = [
+                "enigmail_keepSettingsForReply",
+                "acceptedKeysValid",
+                "acceptedKeysAll",
+                "autoSendEncryptedNever",
+                "autoSendEncryptedIfKeys",
+                "confirmBeforeSendingNever",
+                "confirmBeforeSendingAlways",
+                "confirmBeforeSendingIfEncrypted",
+                "confirmBeforeSendingIfNotEncrypted",
+                "confirmBeforeSendingIfRules",
+              ];
+  var elem;
+  for (var i=0; i < elems.length; ++i) {
+    elem = document.getElementById(elems[i]);
+    if (disable) {
+      elem.setAttribute("disabled","true");
+    }
+    else {
+      elem.removeAttribute("disabled");
+    }
+  }
+}
+
+function updateSendingPrefs()
+{
+  EnigDisplayRadioPref("acceptedKeys", EnigGetPref("acceptedKeys"),
+                       gEnigAcceptedKeys);
+  EnigDisplayRadioPref("autoSendEncrypted", EnigGetPref("autoSendEncrypted"),
+                       gEnigAutoSendEncrypted);
+  EnigDisplayRadioPref("confirmBeforeSending", EnigGetPref("confirmBeforeSending"),
+                       gEnigConfirmBeforeSending);
+  gEnigEncryptionModel = EnigGetPref("encryptionModel");
+  disableManually(gEnigEncryptionModel == 0);
+  displayPrefs(false, true, false);
+}
+
+function resetSendingPrefsConvenient()
+{
+  DEBUG_LOG("pref-enigmail.js: resetSendingPrefsConvenient()\n");
+
+  // save current manual preferences to be able to switch back to them:
+  gSavedManualPrefKeepSettingsForReply = document.getElementById("enigmail_keepSettingsForReply").checked;
+  gSavedManualPrefAcceptedKeys = document.getElementById("enigmail_acceptedKeys").value;
+  gSavedManualPrefAutoSendEncrypted = document.getElementById("enigmail_autoSendEncrypted").value;
+  gSavedManualPrefConfirmBeforeSending = document.getElementById("enigmail_confirmBeforeSending").value;
+
+  // switch encryption model:
+  gEnigEncryptionModel = 0;       // convenient encryption settings
+  EnigSetPref("encryptionModel", gEnigEncryptionModel);
+
+  // update GUI elements and corresponding setting variables:
+  var keepSettingsForReply = true;  // reply encrypted on encrypted emails
+  gEnigAcceptedKeys = 1;            // all keys accepted
+  gEnigAutoSendEncrypted = 1;       // auto.send-encrypted if accepted keys exist
+  gEnigConfirmBeforeSending = 0;    // never confirm before sending
+  EnigSetPref("keepSettingsForReply", keepSettingsForReply);
+  EnigSetPref("acceptedKeys", gEnigAcceptedKeys);
+  EnigSetPref("autoSendEncrypted", gEnigAutoSendEncrypted);
+  EnigSetPref("confirmBeforeSending", gEnigConfirmBeforeSending);
+
+  updateSendingPrefs();
+}
+
+function resetSendingPrefsManually()
+{
+  DEBUG_LOG("pref-enigmail.js: resetSendingPrefsManually()\n");
+
+  // switch encryption model:
+  gEnigEncryptionModel = 1;         // manual encryption settings
+  EnigSetPref("encryptionModel", gEnigEncryptionModel);
+
+  // update GUI elements and corresponding setting variables
+  // with saved old manual preferences:
+  var keepSettingsForReply = gSavedManualPrefKeepSettingsForReply;
+  gEnigAcceptedKeys = gSavedManualPrefAcceptedKeys;
+  gEnigAutoSendEncrypted = gSavedManualPrefAutoSendEncrypted;
+  gEnigConfirmBeforeSending = gSavedManualPrefConfirmBeforeSending;
+  EnigSetPref("keepSettingsForReply", keepSettingsForReply);
+  EnigSetPref("acceptedKeys", gEnigAcceptedKeys);
+  EnigSetPref("autoSendEncrypted", gEnigAutoSendEncrypted);
+  EnigSetPref("confirmBeforeSending", gEnigConfirmBeforeSending);
+
+  updateSendingPrefs();
 }
 
 function resetRememberedValues() {
@@ -274,7 +390,6 @@ function resetRememberedValues() {
              "initAlert",
              "mimePreferPgp",
              "quotedPrintableWarn",
-             "saveEncrypted",
              "warnOnRulesConflict",
              "warnGpgAgentAndIdleTime",
              "warnClearPassphrase",
@@ -310,9 +425,6 @@ function prefOnAccept() {
   displayPrefs(false, false, true);
   EnigSetPref("agentPath", EnigConvertFromUnicode(newAgentPath, "utf-8"));
 
-  //setRecipientsSelectionPref(document.getElementById("enigmail_recipientsSelection").value);
-  //EnigSetRadioPref("recipientsSelection", gEnigRecipientsSelection);
-
   if (gMimePartsElement &&
       (gMimePartsElement.checked != gMimePartsValue) ) {
 
@@ -357,6 +469,9 @@ function prefOnAccept() {
     }
   }
 
+  // update status bar because whether/how to process rules might have changed
+  // NO EFFECT, TB hangs:
+  //Enigmail.msg.updateStatusBar();
 
   return true;
 }
@@ -538,3 +653,4 @@ function enigLocateGpg() {
     document.getElementById("enigmail_agentPath").value = filePath.path;
   }
 }
+
